@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1990 The Regents of the University of California.
+ * Copyright (c) 1990, 2007 The Regents of the University of California.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms are permitted
@@ -31,8 +31,7 @@ ANSI_SYNOPSIS
 
 	#include <stdio.h>
 	size_t _fread_r(struct _reent *<[ptr]>, void *<[buf]>,
-	                size_t <[size]>, size_t <[count]>,
-		        FILE *<[fp]>);
+	                size_t <[size]>, size_t <[count]>, FILE *<[fp]>);
 
 TRAD_SYNOPSIS
 	#include <stdio.h>
@@ -88,7 +87,7 @@ _DEFUN(crlf_r, (ptr, fp, buf, count, eof),
        size_t count _AND
        int eof)
 {
-  int newcount = 0, r;
+  int r;
   char *s, *d, *e;
 
   if (count == 0)
@@ -147,7 +146,8 @@ _DEFUN(_fread_r, (ptr, buf, size, count, fp),
 
   CHECK_INIT(ptr, fp);
 
-  _flockfile (fp);
+  _newlib_flockfile_start (fp);
+  ORIENT (fp, -1);
   if (fp->_r < 0)
     fp->_r = 0;
   total = resid;
@@ -168,7 +168,7 @@ _DEFUN(_fread_r, (ptr, buf, size, count, fp),
 
       /* If still more data needed, free any allocated ungetc buffer.  */
       if (HASUB (fp) && resid > 0)
-	FREEUB (fp);
+	FREEUB (ptr, fp);
 
       /* Finally read directly into user's buffer if needed.  */
       while (resid > 0)
@@ -179,9 +179,9 @@ _DEFUN(_fread_r, (ptr, buf, size, count, fp),
 	  void * old_p = fp->_p;
 	  int old_size = fp->_bf._size;
 	  /* allow __refill to use user's buffer */
-	  fp->_bf._base = p;
+	  fp->_bf._base = (unsigned char *) p;
 	  fp->_bf._size = resid;
-	  fp->_p = p;
+	  fp->_p = (unsigned char *) p;
 	  rc = __srefill_r (ptr, fp);
 	  /* restore fp buffering back to original state */
 	  fp->_bf._base = old_base;
@@ -195,11 +195,11 @@ _DEFUN(_fread_r, (ptr, buf, size, count, fp),
 #ifdef __SCLE
               if (fp->_flags & __SCLE)
 	        {
-	          _funlockfile (fp);
+	          _newlib_flockfile_exit (fp);
 	          return crlf_r (ptr, fp, buf, total-resid, 1) / size;
 	        }
 #endif
-	      _funlockfile (fp);
+	      _newlib_flockfile_exit (fp);
 	      return (total - resid) / size;
 	    }
 	}
@@ -220,11 +220,11 @@ _DEFUN(_fread_r, (ptr, buf, size, count, fp),
 #ifdef __SCLE
 	      if (fp->_flags & __SCLE)
 		{
-		  _funlockfile (fp);
+		  _newlib_flockfile_exit (fp);
 		  return crlf_r (ptr, fp, buf, total-resid, 1) / size;
 		}
 #endif
-	      _funlockfile (fp);
+	      _newlib_flockfile_exit (fp);
 	      return (total - resid) / size;
 	    }
 	}
@@ -237,11 +237,11 @@ _DEFUN(_fread_r, (ptr, buf, size, count, fp),
 #ifdef __SCLE
   if (fp->_flags & __SCLE)
     {
-      _funlockfile (fp);
+      _newlib_flockfile_exit (fp);
       return crlf_r(ptr, fp, buf, total, 0) / size;
     }
 #endif
-  _funlockfile (fp);
+  _newlib_flockfile_end (fp);
   return count;
 }
 
